<?php

/**
 *
 */
function geomap_cck_admin_form() {
  $path_to_module = base_path() . drupal_get_path('module', 'geomap_cck');

  $form = array();
  $group_weight = -20;

  $default_values_resset = db_query("SELECT * FROM {geomap_cck_nodetype_settings}");
  $default_values = array();
  while ($default_values_object = db_fetch_object($default_values_resset)) {
    $default_values[$default_values_object->type] = $default_values_object;
  }

  $form['geomap_cck_node_types'] = array(
    '#type' => 'fieldset',
    '#weight' => $group_weight,
    '#title' => t('Geomap CCK Settings'),
    '#description' => t('Select CCK fields on a per <b>Node Type</b> basis for the Geomap. If a specific node type is not set <b>and</b> the default is not set, then no information will be loaded for the node type.'),
    '#collapsible' => FALSE,
    '#tree' => TRUE,
  );
  $group_weight++;

  // For each node type, the following may be set
  if ($resset = db_query("SELECT 'default' as name, 'default' as type UNION SELECT name, type FROM {node_type} ORDER BY name")) {
    while ($result = db_fetch_object($resset)) {
      if ($result->name == "default") {
        $form['geomap_cck_node_types'][$result->type] = array(
          '#type' => 'fieldset',
          '#weight' => $group_weight,
          '#title' => t('!name settings', array('!name' => strtoupper($result->name))),
          '#collapsible' => TRUE,
          '#collapsed' => FALSE,
        );
        $group_weight++;

        //collect all field names - regardless if the nodes have them or not!
        $options = array('none' => t('- none -'));
        $db_options = db_query("SELECT 'node_title' AS field_name UNION SELECT DISTINCT field_name FROM {content_node_field} c WHERE active = %d ORDER BY field_name", 1);
        while ($db_option = db_fetch_object($db_options)) {
          $options[$db_option->field_name] = strtoupper($db_option->field_name);
        }

      }
      else {
        $form['geomap_cck_node_types'][$result->type] = array(
          '#type' => 'fieldset',
          '#weight' => $group_weight,
          '#title' => t('!name settings', array('!name' => strtoupper($result->name))),
          '#collapsible' => TRUE,
          '#collapsed' => TRUE,
        );
        $group_weight++;

        //collect only the field names that are registered for that node!
        $options = array('none' => t('- none -'));
        $db_options = db_query("SELECT 'node_title' AS field_name, 'Node Title' AS label UNION SELECT field_name, label FROM {content_node_field_instance} c WHERE type_name = '%s' AND widget_active = %d ORDER BY label", $result->name, 1);
        while ($db_option = db_fetch_object($db_options)) {
          $options[$db_option->field_name] = strtoupper($db_option->label);
        }
      }

      $form['geomap_cck_node_types'][$result->type]['latitude'] = array(
        '#type' => 'select',
        '#default_value' => ($default_values[$result->type]->cck_field_latitude === NULL ? 'none' : $default_values[$result->type]->cck_field_latitude),
        '#options' => $options,
        '#weight' => $group_weight,
        '#title' => t('Latitude'),
      );
      $group_weight++;

      $form['geomap_cck_node_types'][$result->type]['longitude'] = array(
        '#type' => 'select',
        '#default_value' => ($default_values[$result->type]->cck_field_longitude === NULL ? 'none' : $default_values[$result->type]->cck_field_longitude),
        '#options' => $options,
        '#weight' => $group_weight,
        '#title' => t('Longitude'),
      );
      $group_weight++;

      $form['geomap_cck_node_types'][$result->type]['window_text'] = array(
        '#type' => 'select',
        '#default_value' => ($default_values[$result->type]->cck_field_window_text === NULL ? 'none' : $default_values[$result->type]->cck_field_window_text),
        '#options' => $options,
        '#weight' => $group_weight,
        '#title' => t('Window Text'),
      );
      $group_weight++;

      $form['geomap_cck_node_types'][$result->type]['display_title'] = array(
        '#type' => 'select',
        '#default_value' => ($default_values[$result->type]->cck_field_display_title === NULL ? 'none' : $default_values[$result->type]->cck_field_display_title),
        '#options' => $options,
        '#weight' => $group_weight,
        '#title' => t('Display Title'),
        '#discription' => t('The dispaly text to be displayed inside of the microformat. See !link for more information.', array('!link' => l('Microformats', 'http://microformats.org/wiki/geo', array('attributes' => array('target' => '_blank'))))),
      );
      $group_weight++;

    }
  }
  $form['#validate'] = array("geomap_cck_admin_form_validate");
  $form['#submit'] = array("geomap_cck_admin_form_submit");

  return system_settings_form($form);
}

function geomap_cck_admin_form_validate($form, &$form_state) {
  //TODO: validate that for each pair, if one value is selected, that the other value is not null!!
  switch ($form_state['values']['op']) {
    case 'Save configuration':
      if (count($form_state['values']['geomap_cck_node_types'])) {
        foreach ($form_state['values']['geomap_cck_node_types'] as $node_type => $fields) {
          if ($fields['latitude'] == "none" || $fields['longitude'] == "none") {
            if ($fields['latitude'] != "none" || $fields['longitude'] != "none") {
              //THROW A VALIDATION ERROR
              form_set_error('geomap_cck_node_types][' . $node_type . '][latitude', t('Incorrect selection for !type', array('!type' => $node_type)));
              form_set_error('geomap_cck_node_types][' . $node_type . '][longitude', ' ');
            }
          }
        }
      }
      break;
    case 'Reset to defaults':
      //todo
      break;
  }
}

/**
 *
 */
function geomap_cck_admin_form_submit($form, &$form_state) {
  /*print "<h2>Form:</h2><pre>";
   print_r($form);
   print "</pre><hr />";
   print "<h2>State:</h2><pre>";
   print_r($form_state);
   print "</pre><hr />";

   $field_latitude = "";
   $field_latitude = "";
   */

  switch ($form_state['values']['op']) {
    case 'Save configuration':
      if (count($form_state['values']['geomap_cck_node_types'])) {
        //Do the fast solution, delete the entire table contents and start fresh.
        db_query("DELETE FROM {geomap_cck_nodetype_settings} WHERE 1");
        foreach ($form_state['values']['geomap_cck_node_types'] as $node_type => $fields) {
          $db_row_values = array(
	        filter_xss($node_type),
	        ($fields['latitude'] == "none" ? NULL : filter_xss($fields['latitude'])),
	        ($fields['longitude'] == "none" ? NULL : filter_xss($fields['longitude'])),
	        ($fields['window_text'] == "none" ? NULL : filter_xss($fields['window_text'])),
	        ($fields['display_title'] == "none" ? NULL : filter_xss($fields['display_title'])),
	      );

          //only update if there are adequate values - should be detected in the form validation as well!
          if ($db_row_values[1] && $db_row_values[2]) {
            db_query("INSERT INTO {geomap_cck_nodetype_settings} VALUES ('%s', '%s', '%s', '%s', '%s')", $db_row_values[0], $db_row_values[1], $db_row_values[2], $db_row_values[3], $db_row_values[4]);
          }
        }
      }
      //Whenever a setting is updated for a specific node type, the table geomap_cck_node should be cleared for that node type.
      //Making things easier on us for now. When the form is submitted, clear the geomap_cck_node table entirely.
      db_query("DELETE FROM {geomap_cck_node} WHERE 1");
      break;
    case 'Reset to defaults':
      //todo
      break;
  }


}
