<?php
/**
 * Theme functions and settings for the geomap module
 */

/**
 * Return the default key specified in the admin screen.
 * By creating a theme function, themers can override this on a per theme basis
 * Add PHP code to the theme function, and the key can be selected on a per domain basis too
 *
 * @ingroup themeable
 * @param $domain
 *   Optionally specify a domain
 * @return $key
 *   A valid Google Maps Key
 * 
 */
function theme_geomap_google_key($domain = null) {
  if(!($domain)){ $domain = $_SERVER['HTTP_HOST']; }
  
  if($key = db_result(db_query("SELECT google_map_key FROM {geomap_googlemap_api_keys} WHERE domain = '%s'", $domain))){
    return $key;
  }
  
  return null;
}

/**
 * Create a map div. 
 * 
 * The Javascript will use this div to render tha map. 
 * This will remain empty if no geo microformats exist on the current page.
 * 
 * @param $delta 
 *   If more than one map needs to be displayed add an identifier
 * @ingroup themeable
 */
function template_preprocess_geomap(&$vars) {
  if (user_access('view geomap') && $google_key = theme('geomap_google_key')) {
    //TODO: Allow a different ID one day (use delta)
    $vars['mapid'] = 'map';
    $vars['classes'] = 'googlemap googlemap-' . $delta;
    $loading = t('loading');
    $vars['loader_image'] = theme('image', drupal_get_path('module', 'geomap') . '/theme/ajax-loader.gif', $loading, $loading);
    $GLOBALS['geomap'] = true;
  } else if (user_access('administer site configuration')) {
	drupal_set_message(l(t('Enter your Google Map Key here.'), 'admin/settings/geomap'), 'warning');
  }
}

/**
 * Theme the info window text
 *
 * @ingroup themeable
 * @param &$vars
 *   See geomap theme. $text (string), $classes (array), $id (string)
 * 
 */
function template_preprocess_geomap_windowtext(&$vars){
  if($vars['text'] === null){ 
    $vars == array(); 
    return; //there is nothing to do if there is no text
  }
  
  $vars['text_id'] = ($vars['text_id'] ? check_plain($vars['text_id']) : null); //no default ID to save on html size
  $vars['text_classes'] = (count($vars['text_classes']) ? "infowindow-text " . implode(' ', $vars['text_classes']) :  "infowindow-text");
  $vars['text'] = filter_xss($vars['text']); //in order for images to be included, use check_markup instead of filter_xss.
}

/**
 * Theme the info window text
 *
 * @ingroup themeable
 * @param &$vars
 *   See geomap_theme. $windowtext (array), $infowindow_classes (array), $infowindow_id (string)
 */
function template_preprocess_geomap_infowindow(&$vars){
  if($vars['windowtext'] === NULL){ 
    unset($vars['infowindow_classes']); 
    unset($vars['infowindow_id']); 
    return; //there is nothing to do if there is no src
  }
  $vars['infowindow_id'] = ($vars['infowindow_id'] ? check_plain($vars['infowindow_id']) : NULL);
  $vars['infowindow_classes'] = (count($vars['infowindow_classes']) ? "infowindow " . implode(' ', $vars['infowindow_classes']) :  "infowindow");
  
  //Text
  $text = $vars['windowtext']['text'];
  $windowtext_classes = $vars['windowtext']['windowtext_classes'];
  $windowtext_id = $vars['windowtext']['windowtext_id'];
  
  $vars['windowtext'] = theme('geomap_windowtext', $text, $windowtext_classes, $windowtext_id);
  //print "<pre>"; print_r($vars); print "</pre>";
}

/**
 * Theme an individual marker based on icon and infowindow
 *
 * Uses template geomap-marker
 *
 * @ingroup themeable
 * @param &$vars
 *   See geomap theme. $src; $icon_classes (array); $icon_id (string); $options (array) => $name (string), $value (string), $classes (array);
 */
function template_preprocess_geomap_icon(&$vars){
  if($vars['src'] === NULL){ 
    unset($vars['src']); 
    unset($vars['icon_classes']); 
    unset($vars['icon_id']); 
    unset($vars['icon_options']); 
    return; //there is nothing to do if there is no src
  }
  
  //Icon
  $vars['icon_id'] = ($vars['icon_id'] ? check_plain($vars['icon_id']) : null);
  $vars['icon_classes'] = (count($vars['icon_classes']) ? "icon " . implode(' ', $vars['icon_classes']) :  "icon");
  $vars['src'] = check_plain($vars['src']);
	
  //Options
  if(is_array($vars['icon_options'])){
    $options = $vars['icon_options'];
    foreach($options as $k => $option){
      //drupal_set_message($option['option_name'] . ": " . $option['option_value']);
	  if($option['option_name'] && $option['option_value']){
        $vars['icon_options'][$k]['option_classes'] = (($option['option_classes']) ? "option " . implode(' ', $option['option_classes']) :  "option");
        $vars['icon_options'][$k]['option_name'] = check_plain($option['option_name']);
        $vars['icon_options'][$k]['option_value'] = check_plain($option['option_value']); //TODO: Do we need to use something else than check_plain?
      } else {
	    unset($vars['icon_options'][$k]);
      }
    }
  }
}

/**
 * Theme an individual marker based on icon and infowindow
 *
 * Uses template geomap-marker
 *
 * @ingroup themeable
 * @param &$vars
 *   See geomap theme. $icon (array), $infowindow (array)
 */
function template_preprocess_geomap_marker(&$vars){
  $vars['marker_classes'] = "marker";
  $vars['marker_id'] = null; //as a default we do not require an id
  
  $icon = $vars['icon'];
  $infowindow = $vars['infowindow'];
  
  //if a custom icon has been sent, then render that, otherwise query the DB for a custom icon
  
  if($icon){
    $vars['icon'] = theme('geomap_icon', $icon['src'], $icon['icon_classes'], $icon['icon_id'], $icon['icon_options']);
  }
  if($infowindow){
    $vars['infowindow'] = theme('geomap_infowindow', $infowindow['windowtext']);
  }
}

/**
 * Theme an individual geolocation based on the input
 *
 * Uses template geomap-geolocation
 *
 * @ingroup themeable
 * @param &$vars
 *   See geomap_theme. $title (string), $latitude (float), $longitude (float), $nid (String),  
 *   $type (String), $path (String), $tids (array), $marker (array)
 * 
 */
function template_preprocess_geomap_geolocation(&$vars){
  
  //NODE:
  if($vars['nid']){
    $vars['nid'] = (int) $vars['nid']; //make certain this is an integer
    $vars['title'] = check_plain($vars['title']); //the node display title may not contain HTML
    $vars['type'] = check_plain($vars['type']); //NOTE: the node type must be a valid drupal type!!!
    //$vars['path'] = "" //TODO: how to check that the path is properly formatted
  } else {
    $vars['nid'] = NULL;
    //we do not care about setting the others to NULL. As long as NID is null, it will not be rendered.
  }
  
  //LAT & LNG COORDINATES
  $vars['latitude'] = (float) $vars['latitude'];
  $vars['longitude'] = (float) $vars['longitude'];
  
  if($vars['latitude'] || $vars['longitude']){
    $vars['latitude_display'] = ($vars['latitude_display'] ? filter_xss($vars['latitude_display']) : NULL);
    $vars['longitude_display'] = ($vars['longitude_display'] ? filter_xss($vars['longitude_display']) : NULL);
  } else {
    //here we assume that 0,0 is not a valid coordinate due to the fact that it is in the middle of the ocean.
    $vars['latitude'] = NULL;
    $vars['longitude'] = NULL;
  }
  
  //MARKER
  $marker_icon = $vars['marker']['icon'];
  $marker_infowindow = $vars['marker']['infowindow'];
  
  $vars['marker'] = NULL;
  
  if($marker_icon === NULL){
	//try the database settings if a marker is set there
	$marker_icon = _geomap_load_icon($vars['type'], $vars['tids']);
  }
  
  if($marker_icon || $marker_infowindow){
    $vars['marker'] = theme('geomap_marker', $marker_icon, $marker_infowindow);
  }
  
  //CSS
  $vars['id'] = NULL; //defaults to null
  $vars['geolocation_classes'] = "geo"; //defaults to geo
}

/**
 * Theme all selected geolocations
 *
 * Uses template geomap-geolocations
 *
 * @ingroup themeable
 * @param &$vars
 *   See geomap_theme. $geolocations (array), $classes (array), $id (string)
 * 
 */
function template_preprocess_geomap_geolocations(&$vars){
  
  $vars['geolocations_classes'] = (count($vars['geolocations_classes']) ? "geolocations " . implode(' ', $vars['geolocations_classes']) : "geolocations");
  $vars['geolocations_id'] = ($vars['geolocations_id'] ? check_plain($vars['geolocations_id']) : NULL);
  
  //copy the array, we replace the $vars['geolocations'] array with the array of rendered objects
  $geolocations = $vars['geolocations'];
  $vars['geolocations'] = array();
  
  foreach($geolocations as $geolocation){
    $vars['geolocations'][] = theme('geomap_geolocation', $geolocation['title'], $geolocation['latitude'], $geolocation['longitude'], $geolocation['nid'], $geolocation['type'], $geolocation['path'], $geolocation['tids'], $geolocation['marker']);
  }
}

/**
 * Helper function to load an icon setting from the database
 *
 * @param $type
 *   Node Type to load
 * @return array
 *   The array for the node type icon
 */
function _geomap_load_icon($node_type = 'page', $tids = array()){
  //There has to be a more efficient way to load the icons. We are
  //querying once currently per node Taxonomy term and node type
  
  //Here the order needs to be remembered! The tids come ordered by vid weight and then by tid weight. 
  //Terms are displayed on the website in order of lowest to highest. Therefore we will load the image
  //for the lowest first. If it is not present, we load the next.
  if(is_array($tids)){
  while($tid = array_shift($tids)){
    if($resultset = db_query("SELECT * FROM {geomap_taxonomy_icon} WHERE tid = %d", $tid)){
      if($result = db_fetch_object($resultset)){
	    $options = array();
	    foreach($result as $k => $value){
		  if(!($k == "icon" || $k == "tid")  && strlen($value)){
			$options[] = array('option_name' => $k, 'option_value' => $value);
		  }
	    }
	    return array(
	      'src' => (strlen($result->icon) ? $result->icon : NULL),
	      'icon_options' => (count($options) ? $options : NULL),
	    );
      }
    }
  }
  }
  
  //no taxonomy specific image was found, load the node type defaults if there is one
  if($resultset = db_query("SELECT * FROM {geomap_nodetype_icon} WHERE type='%s'", $node_type)){
    if($result = db_fetch_object($resultset)){
	  $options = array();
	  foreach($result as $k => $value){
	    if(!($k == "icon" || $k == "type")  && strlen($value)){
		  $options[] = array('option_name' => $k, 'option_value' => $value);
		}
	  }
	  return array(
	    'src' => (strlen($result->icon) ? $result->icon : NULL),
	    'icon_options' => (count($options) ? $options : NULL),
	  );
    }
  }
  
  //no taxonomy specific image was found, load the node type defaults if there is one
  if($resultset = db_query("SELECT * FROM {geomap_nodetype_icon} WHERE type='default'")){
    if($result = db_fetch_object($resultset)){
	  $options = array();
	  foreach($result as $k => $value){
	    if(!($k == "icon" || $k == "type")  && strlen($value)){
		  $options[] = array('option_name' => $k, 'option_value' => $value);
		}
	  }
	  return array(
	    'src' => (strlen($result->icon) ? $result->icon : NULL),
	    'icon_options' => (count($options) ? $options : NULL),
	  );
    }
  }
  
  //load the default
  return NULL;
}